//===========================================================================
// C++ LPC1114 blink-a-led for LPCXpresso LPC1114 : LED on PIO0_7
// Wouter van Ooijen - wouter@voti.nl
// license: zlib
//===========================================================================

#include "LPC11xx.h"

class output_pin {
public:
   virtual void set( bool x ) = 0; 
};

class output_pin_lpc1: public output_pin {
private:
   volatile int *write_pointer;
   
   static volatile int *gpioreg( unsigned int port, unsigned int offset = 0 ){
      return (volatile int *)( 0x50000000 + port * 0x10000 + offset );
   }	
   
public: 
   output_pin_lpc1( unsigned int port, unsigned int pin ){ 
    	*gpioreg( port, 0x8000 ) |= 1 << pin;	 
   	  write_pointer = gpioreg( port, 0x04 << pin );   	  
   }
   
   void set( bool x ){ 
      *write_pointer = x ? -1 : 0;      	
   }
};

class lpc1114 {
public:
   output_pin_lpc1 pin( unsigned int port, unsigned int pin ){
      return output_pin_lpc1( port, pin );
   }
};	

void wait( unsigned int t ){
   while( t > 1000 ){
      wait( 1000 );	
      t -= 1000;
   }	
   SysTick->CTRL = 0;      // stop the timer
   SysTick->LOAD = t * 6;  // CPU clock / ( 2 * 6 ) => 1 MHz
   SysTick->VAL = 0;       // clear the timer
   SysTick->CTRL = 1;      // start the timer
   while( ( SysTick->CTRL & ( 1 << 16 ) ) == 0){} // wait for timeout

}

class hc595;

class output_pin_595 : public output_pin {
private:
   hc595 &chip;
   unsigned int pin;	
public:   	
   output_pin_595( hc595 &chip, unsigned int pin ): 
      chip( chip ), pin( pin ){}	  
   void set( bool x );
};


class hc595 {
private:

   output_pin &hclk, &sclk, &data;    
   unsigned int buffer;   
   
   void flush(){
   	  unsigned int d = buffer;
      for( unsigned int i = 0; i < 8; i++ ){
         data.set( d & 0x80 );
         sclk.set( 0 );
         wait( 1 );
         sclk.set( 1 );
         wait( 1 );
         d = d << 1;
      }   
      hclk.set( 0 );
      wait( 1 );
      hclk.set( 1 );
      wait( 1 );                             
   }
   
public:   
       
   hc595( output_pin &hclk, output_pin &sclk, output_pin &data ):
      hclk( hclk ), sclk( sclk ), data( data ), buffer( 0 ){}       
   
   void set( unsigned int pin, bool x ){
      if( x ){
         buffer |= 1 << pin;   ;  
      } else {
         buffer &= ~ ( 1 << pin );	
      }	
      flush();
   }
   
   output_pin_595 pin( unsigned int n ){
      return output_pin_595( *this, n );   	
   }
   
};

void output_pin_595::set( bool x ){
   chip.set( pin, x );	
}

void blink( output_pin &led, unsigned int delay = 200 * 1000 ){
    for(;;){
       led.set( 1 );
       wait(  delay );
       led.set( 0 );
       wait(  delay );
   }      
};

lpc1114 target;					
int main( void ){		
	output_pin_lpc1 hclk = target.pin( 1, 11 );
	output_pin_lpc1 sclk = target.pin( 2, 11 );
	output_pin_lpc1 data = target.pin( 0, 9 );
	hc595 sr( hclk, sclk, data );
	output_pin_595 led = sr.pin( 2 );
	// output_pin_lpc1 led = target.pin( 0, 7 );
	blink( led );
}
